#!BPY

"""
Name: 'Layer Manager'
Blender: 237a
Group: 'Misc'
Tooltip: 'Layer Management'
"""

__author__ = "FourMadMen.com (FourMadMen)"
__url__ = [
	"FourMadMen.com, http://www.fourmadmen.com",
	"Scripts by FourMadMen, http://www.fourmadmen.com/blender/scripts/index.html",
	"Layer Manager Homepage, http://www.fourmadmen.com/blender/scripts/4mm_layer_manager/index.html",
	"blender",
	"elysiun"
]
__version__ = "1.2 2006/01/23"

__bpydoc__ = """\
This script (Layer Manager) is used to manage scene layers.

With Layer Manager you can name layers, create layer groups, and hide un-used layers in much the same as you can hide mesh geometry.

Terms:

Active Layer: The layer that new objects will be created in.<br>
Available Layer: An availalbe layer is show by Layer Manager and can be made Visible/Invisible and can be selected for purposes of hiding, adding to a layer group, etc.<br>
Selected Layer: An available layers whose selection box is turned on.  Selected layers can be hidden, added to a layer group, etc.<br>
Hidden Layer: A hidden layer is not show in the Layer Manager and can not be selected.  The option to show hidden layers is known as revealing.<br>

Shortcuts:

A: Toggle layer visibility<br>
H: Hide selected layers<br>
R: Reveal hidden layers<br>
S: Toggle layer selection<br>
L: Lock selected layers<br>
U: Unlock selected layers<br>
M: Move selected object to selected layers<br/>
Q: quit<br>
CLTRL+Left Click:  Deselects all other items (works for layer visibility and layer selection<br>

Known Issues:

Blender.Window.Redraw(), which should redraw all 3D Views does not seem to do so.  The 3D View that was most recently usde prior to running the script seems to be the only one redrawn.  To force a redraw click on the Layer lock button.  Upon activation of the lock the view is updated.  Perhaps someone can shed some light on this issue?<br>

Object lock and un-locking is finicky at best due to the (in my opinion) needed re-write of the event/redraw system.  Perhaps when spacehandler scripts play nicely with the undo system this will improve.

Notes:

Future versions may include features such as layer merging, object searching, and (better) layer locking.  The and other features are currently under investigation.<br>
	
"""

# --------------------------------------------------------------------------
# $Id: 4mm_layer_manager.py,v 1.2 2006/01/23 23:55:00
# --------------------------------------------------------------------------
#
# --------------------------------------------------------------------------
# Layer Manager v1.2
# by FourMadMen.com (FourMadMen), 2006
# This script is protected by the GPL: Gnu Public Licence
# GPL - http://www.gnu.org/copyleft/gpl.html
# --------------------------------------------------------------------------
# ***** BEGIN GPL LICENSE BLOCK *****
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software Foundation,
# Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# ***** END GPL LICENCE BLOCK *****
# --------------------------------------------------------------------------

#Press Alt-P to run

import Blender
from Blender import NMesh, Window, Scene, Draw, Types
from Blender.Scene  import *
from Blender.Draw import Register
from Blender.BGL import *
import marshal

######################################################################
# Globals
#

#######################################
#
# User Configuration (Must build GUI for this as I have a few )
#

load_at_startup = True

#
#
#######################################

# Events
EVENT_NOEVENT = 0
EVENT_EXIT = 1
EVENT_HELP = 2
EVENT_SAVE = 3
EVENT_LOAD = 4
EVENT_NEW = 5
EVENT_LAYER_VISIBLE_BASE = 100
EVENT_LAYER_SELECTED_BASE = 200
EVENT_LAYER_NAME_BASE = 300
EVENT_LG_NEW = 21
EVENT_LG_DELETE = 22
EVENT_LG_VIEW = 23
EVENT_LG_ASSIGN = 24
EVENT_LG_REMOVE = 25
EVENT_LG_VIEW_GROUP = 26
EVENT_LG_SELECT = 27
EVENT_LG_DESELECT = 28
EVENT_LG_BROWSE = 29
EVENT_LG_CHANGE = 30
EVENT_SEL_SWAP = 41
EVENT_SEL_HIDE = 42
EVENT_SEL_SHOW = 43
EVENT_SEL_MOVE = 44
EVENT_SEL_LOCK = 45
EVENT_SEL_UNLOCK = 46

LAYER_MANAGER_FILE_EXTENSION = ".4mm.layers"
RELEASE_INFO_KEY = "release_info"
VERSION_KEY = "version"
AUTHOR_KEY = "author"
RELEASE_DATE_KEY = "release_date"
LAYER_PROPERTIES_KEY = "layer_properties"
LAYER_GROUPS_KEY = "layer_groups"
LAYER_SEARCH_KEY = "layer_search"
LAYER_EXTENSIONS_KEY = "layer_extentions"
ACTIVE_KEY = "active"
VISIBLE_KEY = "visible"
SELECTED_KEY = "selected"
HIDDEN_KEY = "hidden"
NAMES_KEY = "names"
ACTIVE_GROUP_KEY = "active_group"
GROUPS_KEY = "groups"
LOCKED_KEY = "locked"
MAIN_KEY = "main"

SEARCH_TYPE_ACTIVE = 1
SEARCH_TYPE_VISIBLE = 2

current_scene = Scene.GetCurrent()

all_layers = [1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20]

def init():
	print "Init Layer Manager"
	
	global layer_info
	
	layer_info = {
		RELEASE_INFO_KEY:{
			VERSION_KEY:"1.0.0",
			AUTHOR_KEY:"FourMadMen.com",
			RELEASE_DATE_KEY:"NYR"
		},
		LAYER_PROPERTIES_KEY:{
			ACTIVE_KEY:1,
			VISIBLE_KEY:Window.ViewLayer()[:],
			SELECTED_KEY:[],
			HIDDEN_KEY:[],
			LOCKED_KEY:[],
			NAMES_KEY:[
				"V1",
				"Layer 1", "Layer 2", "Layer 3", "Layer 4", "Layer 5",
				"Layer 6", "Layer 7", "Layer 8", "Layer 9", "Layer 10",
				"Layer 11", "Layer 12", "Layer 13", "Layer 14", "Layer 15",
				"Layer 16", "Layer 17", "Layer 18", "Layer 19", "Layer 20"
			]
		},
		LAYER_GROUPS_KEY:{
			ACTIVE_GROUP_KEY:0,
			GROUPS_KEY:[
				["Group", []],
			]
		},
		LAYER_SEARCH_KEY:{
			"query_string":"",
			"query_type":SEARCH_TYPE_VISIBLE,
			"results":[]
		},
		LAYER_EXTENSIONS_KEY:{
			"globals":{
				"materials":{},
				"textures":{}
			}
		}
	}
#end init

init()

buttons = {
	LAYER_PROPERTIES_KEY:{
		VISIBLE_KEY: [
			"V1",
			Draw.Create(""), Draw.Create(""), Draw.Create(""), Draw.Create(""), Draw.Create(""),
			Draw.Create(""), Draw.Create(""), Draw.Create(""), Draw.Create(""), Draw.Create(""),
			Draw.Create(""), Draw.Create(""), Draw.Create(""), Draw.Create(""), Draw.Create(""),
			Draw.Create(""), Draw.Create(""), Draw.Create(""), Draw.Create(""), Draw.Create("")
		],
		SELECTED_KEY: [
			"V1",
			Draw.Create(""), Draw.Create(""), Draw.Create(""), Draw.Create(""), Draw.Create(""),
			Draw.Create(""), Draw.Create(""), Draw.Create(""), Draw.Create(""), Draw.Create(""),
			Draw.Create(""), Draw.Create(""), Draw.Create(""), Draw.Create(""), Draw.Create(""),
			Draw.Create(""), Draw.Create(""), Draw.Create(""), Draw.Create(""), Draw.Create("") 
		],
		NAMES_KEY:[
			"V1",
			Draw.Create(""), Draw.Create(""), Draw.Create(""), Draw.Create(""), Draw.Create(""),
			Draw.Create(""), Draw.Create(""), Draw.Create(""), Draw.Create(""), Draw.Create(""),
			Draw.Create(""), Draw.Create(""), Draw.Create(""), Draw.Create(""), Draw.Create(""),
			Draw.Create(""), Draw.Create(""), Draw.Create(""), Draw.Create(""), Draw.Create("")
		]
	},
	LAYER_GROUPS_KEY:[
		Draw.Create(""), Draw.Create(""), Draw.Create(""), Draw.Create(""),
		Draw.Create(""), Draw.Create(""), Draw.Create(""), Draw.Create("")
	],
	SELECTED_KEY:[
		Draw.Create(""), Draw.Create(""), Draw.Create(""),
		Draw.Create(""), Draw.Create(""), Draw.Create("")
	],
	MAIN_KEY:[
		Draw.Create(""), Draw.Create(""), Draw.Create(""),
		Draw.Create(""), Draw.Create("")
	]
}

def toggle_visible_layers():
	hidden_layers = layer_info[LAYER_PROPERTIES_KEY][HIDDEN_KEY]
	available_layers = [layer for layer in all_layers if layer not in hidden_layers]
	visible_layer_count = len(layer_info[LAYER_PROPERTIES_KEY][VISIBLE_KEY])
	
	if visible_layer_count>1:
		visible_layers = [available_layers[0]]
	else:
		visible_layers = available_layers
	#end if
	
	Window.ViewLayer(visible_layers)
	layer_info[LAYER_PROPERTIES_KEY][VISIBLE_KEY] = visible_layers
		
#end toggle_visible_layers

def toggle_layer_visible(layer):
	visible_layers = layer_info[LAYER_PROPERTIES_KEY][VISIBLE_KEY]
	
	if Window.GetKeyQualifiers() & Window.Qual.CTRL:
		visible_layers = [layer]
	else:
		if layer in visible_layers:
			if len(visible_layers)>1: visible_layers.remove(layer)
		else:
			visible_layers.append(layer)
		#end if
	#end if
	
	Window.ViewLayer(visible_layers)
	layer_info[LAYER_PROPERTIES_KEY][VISIBLE_KEY] = visible_layers
	
#end toggle_layer_visible

def view_layer_group():
	active_layer_group = get_layer_group_layers(get_active_layer_group())[:]
	if len(active_layer_group)==0: return
	
	visible_layers = layer_info[LAYER_PROPERTIES_KEY][VISIBLE_KEY]
	
	print active_layer_group
	print visible_layers
	print
	
	for layer in active_layer_group:
		if layer not in visible_layers:
			visible_layers.append(layer)
		#end if
	#end for
	
	Window.ViewLayer(visible_layers)
	layer_info[LAYER_PROPERTIES_KEY][VISIBLE_KEY] = visible_layers
	
#end view_group

def view_layer_group_only():
	active_layer_group = get_layer_group_layers(get_active_layer_group())[:]
	if len(active_layer_group)==0: return
	
	Window.ViewLayer(active_layer_group)
	layer_info[LAYER_PROPERTIES_KEY][VISIBLE_KEY] = active_layer_group
		
#end view_group

def toggle_selected_layers():
	hidden_layers = layer_info[LAYER_PROPERTIES_KEY][HIDDEN_KEY]
	available_layers = [layer for layer in all_layers if layer not in hidden_layers]
	selected_layer_count = len(layer_info[LAYER_PROPERTIES_KEY][SELECTED_KEY])
	
	if selected_layer_count>0:
		selected_layers = []
	else:
		selected_layers = available_layers
	#end if
	
	layer_info[LAYER_PROPERTIES_KEY][SELECTED_KEY] = selected_layers
	
#end toggle_selected_layers

def swap_selected_layers():
	hidden_layers = layer_info[LAYER_PROPERTIES_KEY][HIDDEN_KEY]
	available_layers = [layer for layer in all_layers if layer not in hidden_layers]
	selected_layers = layer_info[LAYER_PROPERTIES_KEY][SELECTED_KEY]
	
	for layer in available_layers:
		if layer in selected_layers:
			selected_layers.remove(layer)
		else:
			selected_layers.append(layer)
		#end if
	#end for
	
	layer_info[LAYER_PROPERTIES_KEY][SELECTED_KEY] = selected_layers
	
#end toggle_selected_layers

def toggle_layer_selected(layer):
	selected_layers = layer_info[LAYER_PROPERTIES_KEY][SELECTED_KEY]
	
	if Window.GetKeyQualifiers() & Window.Qual.CTRL:
		selected_layers = [layer]
		layer_info[LAYER_PROPERTIES_KEY][SELECTED_KEY] = selected_layers
	else:
		if layer in selected_layers:
			selected_layers.remove(layer)
		else:
			selected_layers.append(layer)
		#end if
	#end if
	
#end toggle_layer_selected

def hide_selected_layers():
	hidden_layers = layer_info[LAYER_PROPERTIES_KEY][HIDDEN_KEY]
	available_layers = [layer for layer in all_layers if layer not in hidden_layers]
	if len(available_layers)<=1: return
	
	visible_layers = layer_info[LAYER_PROPERTIES_KEY][VISIBLE_KEY]
	selected_layers = layer_info[LAYER_PROPERTIES_KEY][SELECTED_KEY]
	
	for layer in selected_layers[:]:
		if len(available_layers)>1:
			hidden_layers.append(layer)	
			available_layers.remove(layer)
			selected_layers.remove(layer)
			if layer in visible_layers: visible_layers.remove(layer)
		#end if
	#end for
	
	if len(visible_layers)==0:
		visible_layers = [available_layers[0]]
	#end if
	
	Window.ViewLayer(visible_layers)
	
#end hide_selected_layers

def reveal_hidden_layers():
	layers_to_select = layer_info[LAYER_PROPERTIES_KEY][HIDDEN_KEY][:]
	layer_info[LAYER_PROPERTIES_KEY][HIDDEN_KEY] = []
	layer_info[LAYER_PROPERTIES_KEY][SELECTED_KEY] = layers_to_select
	
#end reveal_hidden_layers():

def save_layer_info():
	default_name = Blender.Get("filename")
	
	if default_name.endswith(".blend"):
	    default_name = default_name[0:len(default_name)-len(".blend")] + LAYER_MANAGER_FILE_EXTENSION
	    
	Window.FileSelector(save_layer_info_fs_callback, "Save Layer Info", default_name)
	
#end save_layer_info

def save_layer_info_fs_callback(file_name):
	data = marshal.dumps(layer_info)
	
	if file_name==None: return
	
	fh = open (file_name, 'w')
	fh.write(data)
	fh.close()
	
#end save_layer_info_fs_callback
	
def load_layer_info(auto_load):
	default_name = Blender.Get("filename")
	
	if default_name.endswith(".blend"):
	    default_name = default_name[0:len(default_name)-len(".blend")] + LAYER_MANAGER_FILE_EXTENSION
	
	if auto_load and default_name!=None:
		#check for errors here, like file not found...
		load_layer_info_fs_callback(default_name)
	else:
		Window.FileSelector(load_layer_info_fs_callback, "Load Layer Info", default_name)	
	#end if
	
#end load_layer_info
	
def load_layer_info_fs_callback(file_name):
	global layer_info
	
	if file_name==None: return
	
	fh = open (file_name)
	data = fh.read()
	fh.close()
	
	saved_layer_info = marshal.loads(data)
	saved_layer_info[RELEASE_INFO_KEY].update(layer_info[RELEASE_INFO_KEY])
	
	layer_info = saved_layer_info
	ensure_compatability(layer_info)
	
	visible_layers = layer_info[LAYER_PROPERTIES_KEY][VISIBLE_KEY]	
	
	Window.ViewLayer(visible_layers[:])
	
#end load_layer_info_fs_callback

def set_layer_name(idx):
	layer_names = layer_info[LAYER_PROPERTIES_KEY][NAMES_KEY]
	layer_name_buttons = buttons[LAYER_PROPERTIES_KEY][NAMES_KEY]
	
	layer_names[idx] = layer_name_buttons[idx].val;
	
#end set_layer_name

def get_layer_groups():
	return layer_info[LAYER_GROUPS_KEY][GROUPS_KEY]
	
#end get_layer_group

def get_layer_group(group):
	return layer_info[LAYER_GROUPS_KEY][GROUPS_KEY][group]
	
#end get_layer_group

def get_active_layer_group():
	return get_layer_group(layer_info[LAYER_GROUPS_KEY][ACTIVE_GROUP_KEY])
	
#end get_active_layer_group

def get_layer_group_name(layer_group):
	return layer_group[0]
	
#end get_layer_group_name

def get_layer_group_layers(layer_group):
	return layer_group[1]
	
#end get_layer_group_layers

def set_active_layer_group_name():
	layer_group_buttons = buttons[LAYER_GROUPS_KEY]
	name = layer_group_buttons[1].val
	
	layer_group = get_active_layer_group()
	layer_group[0] = name
	
#end set_active_layer_group_name

def set_active_layer_group(idx):
	layer_info[LAYER_GROUPS_KEY][ACTIVE_GROUP_KEY] = idx
	
#set_active_layer_group

def delete_active_layer_group():
	active_lg = layer_info[LAYER_GROUPS_KEY][ACTIVE_GROUP_KEY]
	
	layer_info[LAYER_GROUPS_KEY][GROUPS_KEY].pop(active_lg)
	active_lg = active_lg - 1
	if active_lg<0: active_lg = 0
	layer_info[LAYER_GROUPS_KEY][ACTIVE_GROUP_KEY] = active_lg
	
#end delete_active_layer_group

def browse_layer_groups():
	layer_groups = get_layer_groups()
	
	idx = 0
	lg_names = {}
	for layer_group in layer_groups:
		lg_names[layer_group[0]] = idx
		idx = idx + 1
	#end for
	
	list = lg_names.items()
	list.sort()
	
	menuStr = ""
	for item in list:
		menuStr+=item[0]+"%x"+str(item[1])+"|"
	#end for
	
	lgi = Draw.PupMenu(menuStr[:-1])
	
	if lgi>=0:
		set_active_layer_group(lgi)
	#end if
	
#end browse_layer_groups:

def new_layer_group():
	layer_groups = get_layer_groups()
	
	if len(layer_groups)>0:
		layer_group = get_active_layer_group()
		name = auto_name(get_layer_group_name(layer_group), [item[0] for item in layer_groups], "Group")
	else:
		name = "Group"
	#end if
	
	layer_groups.append([name, []])
	set_active_layer_group(len(layer_groups)-1)
	
#end new_layer_group

def assign_selected_to_layer_group():
	hidden_layers = layer_info[LAYER_PROPERTIES_KEY][HIDDEN_KEY]
	selected_layers = layer_info[LAYER_PROPERTIES_KEY][SELECTED_KEY]
	available_layers = [layer for layer in selected_layers if layer not in hidden_layers]
	
	layer_group = get_active_layer_group()
	layer_group_layers = layer_group[1]
	
	for layer in available_layers:
		if layer not in layer_group_layers:
			layer_group_layers.append(layer)
		#end if
	#end for
	
#end assign_selected_to_layer_group

def remove_selected_from_layer_group():
	hidden_layers = layer_info[LAYER_PROPERTIES_KEY][HIDDEN_KEY]
	selected_layers = layer_info[LAYER_PROPERTIES_KEY][SELECTED_KEY]
	available_layers = [layer for layer in selected_layers if layer not in hidden_layers]
	
	layer_group = get_active_layer_group()
	layer_group_layers = layer_group[1]
	
	for layer in available_layers:
		if layer in layer_group_layers:
			layer_group_layers.remove(layer)
		#end if
	#end for
	
#end assign_selected_to_layer_group

def select_layers_in_group(state):
	
	hidden_layers = layer_info[LAYER_PROPERTIES_KEY][HIDDEN_KEY]
	layer_group = get_active_layer_group()
	layer_group_layers = layer_group[1]
	
	selected_layers = layer_info[LAYER_PROPERTIES_KEY][SELECTED_KEY]
	layers_to_toggle = [layer for layer in layer_group_layers if layer not in hidden_layers]
	
	for layer in layers_to_toggle:
		toggle_it = False
		
		if state == True:
			if layer not in selected_layers: toggle_it = True
		else:
			if layer in selected_layers: toggle_it = True
		#end if
		
		if toggle_it: toggle_layer_selected(layer)
	#end for
	
#end assign_selected_to_layer_group

def move_to_selected_layers():
	selected_layers = layer_info[LAYER_PROPERTIES_KEY][SELECTED_KEY][:]
	if len(selected_layers)==0: return
	
	selected_objects = Blender.Object.GetSelected()
	
	for object in selected_objects:
		object.layers = selected_layers
	#end for
	
#end move_to_layer

def lock_selected_layers():
	selected_layers = layer_info[LAYER_PROPERTIES_KEY][SELECTED_KEY]
	locked_layers = layer_info[LAYER_PROPERTIES_KEY][LOCKED_KEY]
	
	for layer in selected_layers:
		if layer not in locked_layers:
			locked_layers.append(layer)
		#end if
	#end for
	
#end lock_selected_layers

def unlock_selected_layers():
	selected_layers = layer_info[LAYER_PROPERTIES_KEY][SELECTED_KEY]
	locked_layers = layer_info[LAYER_PROPERTIES_KEY][LOCKED_KEY]
	
	for layer in selected_layers:
		if layer in locked_layers:
			locked_layers.remove(layer)
		#end if
	#end for
	
#end unlock_selected_layers

def ensure_compatability(layer_info):
	print "Checking loaded information..."
	
	if LOCKED_KEY not in layer_info[LAYER_PROPERTIES_KEY]:
		print "Adding new structure:",LAYER_PROPERTIES_KEY,"->",LOCKED_KEY
		layer_info[LAYER_PROPERTIES_KEY][LOCKED_KEY] = []
	#end if
	
	print "Check complete."
#end ensure_compatability

def check_locked_objects():
	locked_layers = layer_info[LAYER_PROPERTIES_KEY][LOCKED_KEY]
	need_redraw = 0
	
	if len(Blender.Object.GetSelected())>0 and len(locked_layers)>0:
		##print "have selected object AND locked layers"
		
		selected_objects = Blender.Object.GetSelected()
		for object in selected_objects:
			##print "object:",object,"is selected"
			for layer in object.layers:
				if layer in locked_layers:
					##print "object:",object,"is locked"
					object.select(0)
					need_redraw = 1
					break
				#end if
			else:
				#fell through, object is not in locked layer
				##print "object:",object,"is not locked"
				pass
			#end for
		#end for
	#end if
	
	return need_redraw
	
#end check_locked_objects
####################################################################################

def decode(*arguments):
	if len(arguments) < 3:
		raise TypeError, 'decode() takes at least 3 arguments (%d given)' % (len(arguments))
	#end if
	
	dict = list(arguments[1:])
	
	if arguments[0] in dict:
		index = dict.index(arguments[0]);
		if index % 2 == 0 and len(dict) > index+1:
			return dict[index+1]
		#end if
		
		return dict[-1]
	elif len(dict) % 2 != 0:
		return dict[-1]
	#end if
	
#def decode

def dictSort(d):
	""" returns a dictionary sorted by keys """
	
	list = d.items()
	list.sort()
	##print list
	k = {}   
	for item in list:
		k[item[0]] = item[1]
	#end for
	
	return k
	
#end dictSort

def auto_name(s, l, dflt):
	i = 0
	j = 0
	
	name = dflt
	
	while name in l:
		#Unlikely to happen but just in case...
		if i==999:
			j += 1
			dflt += "." + str(j)
		#end if
		
		i += 1
		
		name = dflt+".%03d" % i
	#end while
	
	return name
	
#end auto_name
####################################################################################

def event(evt, val):
	if evt>0: # and not val:
		need_redraw = check_locked_objects()
		if need_redraw:
			Draw.Redraw(1)
		#end if
	#end if
	
	if ((evt == Draw.QKEY) and not val):
		Draw.Exit()
	elif evt == Draw.HKEY and val:
		hide_selected_layers()
	elif evt == Draw.RKEY and val:
		reveal_hidden_layers()
	elif evt == Draw.SKEY and val:
		toggle_selected_layers()
	elif evt == Draw.AKEY and val:
		toggle_visible_layers()
	elif evt == Draw.LKEY and val:
		lock_selected_layers()
	elif evt == Draw.UKEY and val:
		unlock_selected_layers()
	elif evt == Draw.MKEY and val:
		move_to_selected_layers()
	else:
		return
	#end if
	
	Window.Redraw(Window.Types.SCRIPT)
	Window.Redraw()
	
#end event

def button_event(evt):
	if evt>0:
		need_redraw = check_locked_objects()
		if need_redraw:
			Draw.Redraw(1)
		#end if
	#end if
	
	if evt == EVENT_EXIT :
		Draw.Exit()
	elif evt == EVENT_HELP :
		Blender.ShowHelp("4mm_layer_manager.py")
	elif evt == EVENT_SAVE :
		save_layer_info()
	elif evt == EVENT_LOAD :
		load_layer_info(False)
	elif evt == EVENT_NEW :
		init()
	elif evt == EVENT_LG_BROWSE :
		browse_layer_groups()
	elif evt == EVENT_LG_CHANGE :
		set_active_layer_group_name()
	elif evt == EVENT_LG_NEW :
		new_layer_group()
	elif evt == EVENT_LG_DELETE :
		delete_active_layer_group()
	elif evt == EVENT_LG_VIEW:
		view_layer_group()
	elif evt == EVENT_LG_ASSIGN :
		assign_selected_to_layer_group()
	elif evt == EVENT_LG_REMOVE :
		remove_selected_from_layer_group()
	elif evt == EVENT_LG_VIEW_GROUP:
		view_layer_group_only()
	elif evt == EVENT_LG_SELECT :
		select_layers_in_group(True)
	elif evt == EVENT_LG_DESELECT :
		select_layers_in_group(False)
	elif evt == EVENT_SEL_SWAP :
		swap_selected_layers()
	elif evt == EVENT_SEL_HIDE :
		hide_selected_layers()
	elif evt == EVENT_SEL_SHOW :
		reveal_hidden_layers()
	elif evt == EVENT_SEL_MOVE :
		move_to_selected_layers()
	elif evt == EVENT_SEL_LOCK :
		lock_selected_layers()
	elif evt == EVENT_SEL_UNLOCK :
		unlock_selected_layers()
	elif evt>EVENT_LAYER_VISIBLE_BASE and evt<EVENT_LAYER_VISIBLE_BASE+100 :
		toggle_layer_visible(evt - EVENT_LAYER_VISIBLE_BASE)
	elif evt>EVENT_LAYER_SELECTED_BASE and evt<EVENT_LAYER_SELECTED_BASE+100 :
		toggle_layer_selected(evt - EVENT_LAYER_SELECTED_BASE)
	elif evt>EVENT_LAYER_NAME_BASE and evt<EVENT_LAYER_NAME_BASE+100 :
		set_layer_name(evt - EVENT_LAYER_NAME_BASE)
	else:
		return
	#end if
	
	Window.Redraw(Window.Types.SCRIPT)
	Window.Redraw()
	
#end button_event

def gui():
	global buttons

	glClear(GL_COLOR_BUFFER_BIT)
	
	layers = Window.ViewLayer()
	
	layer_names = layer_info[LAYER_PROPERTIES_KEY][NAMES_KEY]
	selected_layers = layer_info[LAYER_PROPERTIES_KEY][SELECTED_KEY]
	hidden_layers = layer_info[LAYER_PROPERTIES_KEY][HIDDEN_KEY]
	locked_layers = layer_info[LAYER_PROPERTIES_KEY][LOCKED_KEY]
	
	layer_visible_buttons = buttons[LAYER_PROPERTIES_KEY][VISIBLE_KEY]
	layer_selection_buttons = buttons[LAYER_PROPERTIES_KEY][SELECTED_KEY]
	layer_name_buttons = buttons[LAYER_PROPERTIES_KEY][NAMES_KEY]
	layer_group_buttons = buttons[LAYER_GROUPS_KEY]
	layer_selected_buttons = buttons[SELECTED_KEY]
	main_buttons = buttons[MAIN_KEY]
	
	section_divider_height = 10
	section_divider_width = 5
	
	start_x = 10
	start_y = 230
	
	lvb_width = 20
	lsb_width = 20
	lnb_width = 100
	lnb_max_input_length = 15
	b_height = 20
	
	ctr = 0
	pos_x = start_x
	pos_y = start_y
	
	column_width = lvb_width + lsb_width + lnb_width + section_divider_width
	
	for i in range(1,21):
		pos_x = start_x
		
		hidden = hidden_layers.count(i)
		
		if hidden==0:
			lv_count = layers.count(i)			
			layer_visible_buttons[i] = Draw.Toggle(str(i), EVENT_LAYER_VISIBLE_BASE+i, pos_x, pos_y,  lvb_width, b_height, lv_count, decode(lv_count, 0, "Make Layer Visible", "Make Layer Invisible"))
			pos_x = pos_x + lvb_width;
			
			ls_count = selected_layers.count(i)
			if i in locked_layers:
				selection_decoration = "L"
			else:
				selection_decoration = " "
			#end if
			
			layer_selection_buttons[i] = Draw.Toggle(selection_decoration, EVENT_LAYER_SELECTED_BASE+i, pos_x, pos_y,  lsb_width, b_height, ls_count, decode(ls_count, 0, "Select Layer", "De-select Layer"))
			pos_x = pos_x + lsb_width;
			
			layer_name_buttons[i] = Draw.String("", EVENT_LAYER_NAME_BASE+i, pos_x, pos_y, lnb_width, b_height, layer_names[i], lnb_max_input_length, "Layer Name")
			pos_x = pos_x + lnb_width;
			
			ctr = ctr + 1
			pos_y = pos_y - b_height
		
			if ctr==5:
				pos_y = pos_y - section_divider_height
			elif ctr==10:
				ctr = 0
				start_x = pos_x + section_divider_width
				pos_y = start_y
			#end if
		#end if
	#end for
	
	main_buttons[0] = Draw.Button("Save",EVENT_SAVE, 10, 10, 50, b_height, "Save layer information")
	main_buttons[1] = Draw.Button("Load",EVENT_LOAD, 70, 10, 50, b_height, "Load layer information")
	main_buttons[2] = Draw.Button("New",EVENT_NEW,  130, 10, 50, b_height, "Reset all layer information to default")
	main_buttons[3] = Draw.Button("Help",EVENT_HELP, 190, 10, 50, b_height, "View Mayer Manager help documentation")
	main_buttons[4] = Draw.Button("Exit",EVENT_EXIT, 250, 10, 50, b_height, "Exit Mayer Manager")
	
	lg_count = len(get_layer_groups())
	
	if lg_count>0:
		layer_group_buttons[0] = Draw.Toggle(" ",             EVENT_LG_BROWSE, 10, 320,  15, b_height, 0, "Browse availale layer groups")
		layer_group_buttons[1] = Draw.String("",              EVENT_LG_CHANGE, 25, 320, 165, b_height, get_layer_group_name(get_active_layer_group()), 25, "Displays current layer group name.  Click to change.")
		layer_group_buttons[3] = Draw.PushButton("Delete",    EVENT_LG_DELETE, 70, 300, 60, b_height, "Removes the current layer group")
		layer_group_buttons[3] = Draw.PushButton("View",      EVENT_LG_VIEW, 130, 300, 60, b_height, "Adds the layers in group to visible layer list")
		layer_group_buttons[4] = Draw.PushButton("Assign",    EVENT_LG_ASSIGN, 10, 280, 60, b_height, "Assign selected layers to the current layer group")
		layer_group_buttons[5] = Draw.PushButton("Remove",    EVENT_LG_REMOVE, 70, 280, 60, b_height, "Remove selected layers from the current layer group")
		layer_group_buttons[5] = Draw.PushButton("View Grp.", EVENT_LG_VIEW_GROUP, 130, 280, 60, b_height, "View only layers that are in the layer group")
		layer_group_buttons[6] = Draw.PushButton("Select",    EVENT_LG_SELECT, 10, 260, 60, b_height, "Selects layers belonging to the current layer group")
		layer_group_buttons[7] = Draw.PushButton("Desel.",    EVENT_LG_DESELECT, 70, 260, 60, b_height, "Deselects layers belonging to the current layer group")
	#end if
	layer_group_buttons[2] = Draw.PushButton("New",    EVENT_LG_NEW, 10, 300, 60, b_height, "Creates a new layer group")
	
	layer_selected_buttons[0] = Draw.PushButton("Select Swap", EVENT_SEL_SWAP, 200, 300, 80, b_height, "Selected unselected layers, and deselects selected layers")
	layer_selected_buttons[1] = Draw.PushButton("Hide",        EVENT_SEL_HIDE, 200, 280, 80, b_height, "Hides selected layers")		
	layer_selected_buttons[2] = Draw.PushButton("Reveal",      EVENT_SEL_SHOW, 200, 260, 80, b_height, "Reveals hidden layers")
	layer_selected_buttons[3] = Draw.PushButton("Move To",     EVENT_SEL_MOVE, 280, 300, 80, b_height, "Moves the selected objects to the selected layers")
	layer_selected_buttons[4] = Draw.PushButton("Lock",        EVENT_SEL_LOCK, 280, 280, 80, b_height, "Lock selected layers")
	layer_selected_buttons[5] = Draw.PushButton("Unlock",      EVENT_SEL_UNLOCK, 280, 260, 80, b_height, "Unlock selected layers")
	
#end gui

if load_at_startup == True:
	try:
		load_layer_info(True)
	except:
		pass
	#end try
#end if
	
Draw.Register(gui, event, button_event)

if load_at_startup == True:
	Window.Redraw(Window.Types.SCRIPT)
	Window.Redraw()
#end if
